#include "UIManager.h"
#include "VideoPage.h"
#include "UserPage.h"
#include "VideoRenderWnd.h"
#include "VideoLayoutFormat.h"
#include "QtGui/qpainter.h"
#include "InviteDlg.h"
#include "MessageBoxFactory.h"
#include "../roomprofile.h"

///
CVideoPage::CVideoPage(RoomWindow* parent, int width, int height) : QWidget(parent) {
	initMember(parent, width, height);
}
CVideoPage::CVideoPage(RoomWindow* parent) : QWidget(parent) {
	initMember(parent, 0, 0);
}
void CVideoPage::initMember(RoomWindow* parent, int width, int height) {
	m_video = NULL;
	m_videoDevice = NULL;
	m_record = NULL;
	m_live = NULL;
	m_livecast = NULL;
	_room_wnd = parent;
	_render_layout_format = new CVideoOneCenterLayoutFormat(width, height);
	_black_boad_width = 0;
	_black_boad_heigth = 0;
#ifdef  __BUILD_USE_FAKECAPTURER__
	_fakecapturer = NULL;
	_timer_fakecapturer = 0;
	_buf_frame = NULL;
	_buf_size = 0;
	_time_stamp_ns = 0;
	_ui_main_thread = QThread::currentThreadId();
	connect(this, SIGNAL(onstart_s()), this, SLOT(OnStart()));
	connect(this, SIGNAL(onstop_s()), this, SLOT(OnStop()));
#endif//__BUILD_USE_FAKECAPTURER__
}
CVideoPage::~CVideoPage() {
#ifdef __BUILD_USE_FAKECAPTURER__
	if (_fakecapturer) {
		FakeVideoCapturer::Destroy(_fakecapturer);
		_fakecapturer = NULL;
	}
#endif
	m_record->setListener(NULL);
	m_live->setListener(NULL);
	ID2ViewType::iterator it = _video_id2view.begin();
	for (; _video_id2view.end() != it;) {
		if (it->second) {
			delete it->second;
		}
		_video_id2view.erase(it++);
	}
}
///
void CVideoPage::AddVideoRender(const UserId& userId, const DeviceId& deviceId) {
	QString renderId = QString::fromUtf8(deviceId.c_str());
	ID2ViewType::iterator it = _video_id2view.find(renderId);
	if (_video_id2view.end() != it) {
		return;
	}
	const User& auser = m_video->getUserManager()->getUser(userId);
	if (!auser.isValid()) {
		return;
	}
	CVideoRenderWnd* render_wnd = new CVideoRenderWnd(renderId, QString::fromUtf8(auser.userName.c_str()), this);
	render_wnd->SetScalingType(CRoomProfile::Instance()->GetRenderType());
	connect(render_wnd, SIGNAL(recordStart(const QString&)), this, SLOT(recordStart(const QString&)));
	connect(render_wnd, SIGNAL(recordStop(const QString&)), this, SLOT(recordStop(const QString&)));
	connect(render_wnd, SIGNAL(liveStart(const QString&)), this, SLOT(liveStart(const QString&)));
	connect(render_wnd, SIGNAL(liveStop(const QString&)), this, SLOT(liveStop(const QString&)));
	render_wnd->show();
	if (m_video->isSelfUser(userId)) {
		if (hasRecord(deviceId)) {
			render_wnd->enableRecord(true);
			if (_currRecordInfo.isValid()) {
				QString qid = QString::fromUtf8(_currRecordInfo.id.c_str());
				render_wnd->setRecordId(qid);
			}
		}
		if (hasLive(deviceId)) {
			render_wnd->enableLive(true);
			if (_currLiveInfo.isValid()) {
				QString qid = QString::fromUtf8(_currLiveInfo.id.c_str());
				render_wnd->setLiveId(qid);
			}
		}
	}
	if (render_wnd->GetRender()) {
		m_video->attachRender(deviceId, render_wnd->GetRender());
	}
	if (_video_id2view.empty() /*|| auser.role == User::host*/) {
		_render_layout_format->SetCenter(render_wnd);
		render_wnd->setBorder(false);
	}
	_video_id2view[renderId] = render_wnd;
	_render_layout_format->AddRenderWnd(renderId, render_wnd);
	_render_layout_format->Layout();
}
void CVideoPage::RemoveVideoRender(const User& usr) {
	CamerasType items;
	m_videoDevice->getRemoteCameras(usr.userId, items);
	if (!items.empty()) {
		RemoveVideoRender(usr.userId, items.front().id);
	}
}
void CVideoPage::RemoveVideoRender(const UserId& userId, const DeviceId& deviceId) {
	QString renderId = QString::fromUtf8(deviceId.c_str());
	m_video->detachRender(deviceId);
	_render_layout_format->RemoveRenderWnd(renderId);
	CVideoRenderWnd* render = _video_id2view[renderId];
	_video_id2view.erase(renderId);
	if (NULL == render) {
		return;
	}
	if (!render->getRecordId().isEmpty()) {
		// continue record when next open
		//	recordStop(render->getRecordId());
	}
	if (!render->getLiveId().isEmpty()) {
		// continue live when next open
		//	liveStop(render->getLiveId());
	}
	if (render != NULL) {
		delete render;
	}
	_render_layout_format->Layout();
}

void CVideoPage::UpdateAllRenderScalingType(tee3::avd::IVideoRenderView::ScalingType type) {
	for (ID2ViewType::iterator it = _video_id2view.begin(); _video_id2view.end() != it; it++) {
		if (it->second != NULL) {
			it->second->SetScalingType(type);
		}
	}
}
void CVideoPage::paintEvent(QPaintEvent *event) {
	QPainter dc(this);
	dc.fillRect(rect(), QBrush(QColor(51, 51, 51)));
	if (_video_id2view.empty()) {
		dc.drawText(rect(), Qt::AlignCenter, QSTRING("ûƵ"));
	}
}
void CVideoPage::resizeEvent(QResizeEvent * event) {
	int width = rect().width();
	int height = rect().height();
	if (width != _black_boad_width || height != _black_boad_heigth) {
		_black_boad_heigth = height;
		_black_boad_width = width;
		_render_layout_format->ChangeSizeNotice(_black_boad_width, _black_boad_heigth);
	}
}
void CVideoPage::RegistVideoHandle(IMVideo* video, IMLivecast* livecast) {
	if (video != NULL) {
		m_video = video;
		m_video->setListener(this);
		m_videoDevice = m_video->getCameraDevice();
		m_videoDevice->setListener(this);
		if (NULL != livecast) {
			m_livecast = livecast;
			m_livecast->setListener(this);
		}
	}
	m_record = IAVDRecord::getRecord(IAVDEngine::Instance());
	m_record->setListener(this);
	m_live = IAVDLive::getLive(IAVDEngine::Instance());
	m_live->setListener(this);
}
#ifdef __BUILD_USE_FAKECAPTURER__
bool CVideoPage::isMainThread() const {
	return _ui_main_thread == QThread::currentThreadId();
}

bool CVideoPage::OnStart() {
	if (!isMainThread())
	{
		emit onstart_s();
		return true;
	}
	_time_stamp_ns = 0;
	_timer_fakecapturer = startTimer(50);
	return true;
}
void CVideoPage::OnStop() {
	if (!isMainThread())
	{
		emit onstop_s();
		return;
	}
	if (_timer_fakecapturer != 0) {
		killTimer(_timer_fakecapturer);
		_timer_fakecapturer = 0;
	}
}
void CVideoPage::timerEvent(QTimerEvent * event)
{
	if (NULL != _fakecapturer && _fakecapturer->isRunning()) {
		uint32 format = FOURCC_I420;
		int w = 1920;
		int h = 1080;
		if (_buf_size != IVideoFrame::SizeOf(w, h)) {
			_buf_size = IVideoFrame::SizeOf(w, h);
			delete[] _buf_frame;
			_buf_frame = new uint8[_buf_size];
			FillMemory(_buf_frame, w * h, 128);
		}

		static int red = 0;
		if (red > 255) {
			red = 0;
		}
		FillMemory(_buf_frame + w * h, w * h / 4, red);
		FillMemory(_buf_frame + w * h + w * h / 4, w * h / 4, red);
		++red;

		int64 elapsed_time_ns = 0;
		_time_stamp_ns += 55;
		/*rtcapi::IVideoFrame* frame = rtcapi::IVideoFrame::Create(format,
			w, h, w, h, _buf_frame, _buf_size, 1, 1, elapsed_time_ns, _time_stamp_ns, 0);
			if (NULL != frame) {
			_fakecapturer->inputCapturedFrame(*frame);
			}*/
		_fakecapturer->inputCapturedFrame(0, format, w, h, _buf_frame, _buf_size, 0, false);
	}
}
#endif

bool CVideoPage::OpenVideo(const UserId& userId) {
	if (userId == m_video->getUserManager()->getSelfUserId()) {
		return OpenVideo();
	}
	else {
		CamerasType cameralist;
		m_videoDevice->getRemoteCameras(userId, cameralist);
		if (!cameralist.empty()) {
			m_video->remotecmdPublishCamera(cameralist.front());
		}
	}
	return false;
}
bool CVideoPage::OpenVideo() {
	const Camera& cam = m_videoDevice->getDefaultCamera();
	if (!cam.isValid()) {
		return false;
	}
	Result ret = Err_Failure;
	Camera publishCam;
#ifdef __BUILD_USE_FAKECAPTURER__
	if (_fakecapturer) {
		FakeVideoCapturer::Destroy(_fakecapturer);
		_fakecapturer = NULL;
	}
	_fakecapturer = FakeVideoCapturer::Create(this);
	publishCam.id = "fakecapturertest";
	publishCam.publishedQualities.setStreamOptions(stream_main, CameraCapability(1280, 720, 15), codec_vp8);
	//publishCam.publishedQualities.setStreamOptions(stream_assist1, CameraCapability(640, 360, 15), codec_vp8);

	// for preview
	QString renderId = QSTRING("fakecapturertest");
	CVideoRenderWnd* render_wnd = new CVideoRenderWnd(renderId, QSTRING("preview fake capturer"), this);
	render_wnd->SetScalingType(CRoomProfile::Instance()->GetRenderType());
	render_wnd->show();
	if (_video_id2view.empty() /*|| auser.role == User::host*/) {
		_render_layout_format->SetCenter(render_wnd);
		render_wnd->setBorder(false);
	}
	_video_id2view[renderId] = render_wnd;
	_render_layout_format->AddRenderWnd(renderId, render_wnd);
	_render_layout_format->Layout();
	//ret = m_video->previewLocalCamera(publishCam, _fakecapturer, render_wnd->GetRender());
	ret = m_video->publishLocalCamera(publishCam, _fakecapturer);
#else
	publishCam = cam;
	bool useDefault = false;
	int  testcase = 9;
	switch (testcase) {
	case 1:
	{ //һͷ3·Ƶh264 720p1 vp8 480p2 h264 320ֱʣ
			  publishCam.level = 20;
			  publishCam.description = "wowo";
			  publishCam.publishedQualities.setStreamOptions(stream_main, CameraCapability(1280, 720, 15), codec_h264);
			  publishCam.publishedQualities.setStreamOptions(stream_assist1, CameraCapability(640, 480, 15), codec_vp8);
			  publishCam.publishedQualities.setStreamOptions(stream_assist2, CameraCapability(320, 240, 15), codec_h264);
	}
		break;
	case 2:
	{ //һͷ3·Ƶvp8 ͨ1 h264 ͨ2 vp8 
			  publishCam.publishedQualities.setRatio(VideoRatio::ratio_16_9);
			  publishCam.publishedQualities.setStreamOptions(stream_main, VideoQuality::quality_normal, codec_vp8);
			  publishCam.publishedQualities.setStreamOptions(stream_assist1, VideoQuality::quality_normal, codec_h264);
			  publishCam.publishedQualities.setStreamOptions(stream_assist2, VideoQuality::quality_low, codec_vp8);
	}
		break;
	case 3:
	{ //һͷ3·Ƶȱʡ뷽ʽ߿16:9 ͨ1 壻2 
			  publishCam.publishedQualities.setRatio(VideoRatio::ratio_16_9);
			  publishCam.publishedQualities.setStreamOptions(stream_main, CameraCapability(640, 360, 15));
			  publishCam.publishedQualities.setStreamOptions(stream_assist1, VideoQuality::quality_high);
			  publishCam.publishedQualities.setStreamOptions(stream_assist2, VideoQuality::quality_low);
	}
		break;
	case 4:
	{ //һͷ2·Ƶvp8 480p1 vp8 320ֱʣ
			  //publishCam.publishedQualities.setStreamOptions(stream_main, CameraCapability(480, 640, 30), codec_vp8);
			  publishCam.publishedQualities.setStreamOptions(stream_main, CameraCapability(180, 320, 30), codec_vp8);
	}
		break;
	default:
		useDefault = true;
		break;
	}
	if (useDefault) {
		if (CRoomProfile::vp_2_codec == CRoomProfile::Instance()->GetVideoPubType()) {
			//publishCam.publishedQualities.setStreamOptions(stream_main, VideoQuality::quality_normal, codec_vp8);
			publishCam.publishedQualities.setStreamOptions(stream_main, VideoQuality::quality_normal, codec_h264);
			ret = m_video->publishLocalCamera(publishCam);
		}
		else {
			ret = m_video->publishLocalCamera();
		}
	}
	else {
		ret = m_video->publishLocalCamera(publishCam);
	}
#endif//__BUILD_USE_FAKECAPTURER__
	_localVideoId = publishCam.id;
	CameraCapability cap = m_video->getRealCaptureCapability(_localVideoId);
	return RT_SUCCEEDED(ret);
}
bool CVideoPage::SwitchCamera() {
	if (_localVideoId.empty()) {
		return false;
	}
	Camera item = m_videoDevice->getCamera(_localVideoId);
	CamerasType cameraList;
	m_videoDevice->getLocalCameras(cameraList);

	for (CamerasType::iterator it = cameraList.begin(); it != cameraList.end(); it++) {
		if (it->id != _localVideoId) {
			item = (*it);
		}
	}
	if (RT_SUCCEEDED(m_video->switchLocalCamera(_localVideoId.c_str(), item.id))) {
		_video_id2view[item.id.c_str()] = _video_id2view[_localVideoId.c_str()];
		_video_id2view[item.id.c_str()]->setId(item.id.c_str());
		_video_id2view.erase(_localVideoId.c_str());
		_localVideoId = item.id;
	}
	return true;
}
void CVideoPage::UpdateCameraData(int level, const std::string& description) {
	m_video->updateCameraData(_localVideoId, level, description);
}
bool CVideoPage::CloseVideo(const UserId& userId) {
	if (userId == m_video->getUserManager()->getSelfUserId()) {
		return CloseVideo();
	}
	else {
		CamerasType cameralist;
		m_videoDevice->getRemoteCameras(userId, cameralist);

		if (!cameralist.empty()) {
			m_video->remotecmdUnpublishCamera(cameralist[0].id);
		}
	}
	return false;
}
bool CVideoPage::CloseVideo() {
	Result ret = Err_Failure;
	if (_localVideoId.empty()) {
		ret = m_video->unpublishLocalCamera();
	}
	else {
		ret = m_video->unpublishLocalCamera(_localVideoId);
		_localVideoId.clear();
	}
	return RT_SUCCEEDED(ret);
}
///
void CVideoPage::onPublishCameraNotify(const Camera& camera) {
	if (!m_video->isSelfUser(m_video->getOwnerId(camera.id))) {
		m_video->subscribe(camera.id);
	}
}
void CVideoPage::onUnpublishCameraNotify(const Camera& camera) {
	if (!m_video->isSelfUser(m_video->getOwnerId(camera.id))) {
		m_video->unsubscribe(camera.id);
		RemoveVideoRender(m_video->getOwnerId(camera.id), camera.id);
	}
}
void CVideoPage::onCameraStatusNotify(CameraStatus status, const DeviceId& fromId) {
}
void CVideoPage::onCameraDataNotify(int level, const String& description, const DeviceId& fromId) {
}
void CVideoPage::onSubscribeResult(Result result, const DeviceId& fromId) {
	if (RT_SUCCEEDED(result)) {
		AddVideoRender(m_video->getOwnerId(fromId), fromId);
	}
}
void CVideoPage::onUnsubscribeResult(Result result, const DeviceId& fromId) {
	if (RT_SUCCEEDED(result)) {
		RemoveVideoRender(m_video->getOwnerId(fromId), fromId);
		_room_wnd->SetVideoState(false);
	}
}
void CVideoPage::onPublishLocalResult(Result result, const DeviceId& fromId) {
	if (RT_SUCCEEDED(result)) {
		AddVideoRender(m_video->getOwnerId(fromId), fromId);
		_room_wnd->SetVideoState(true);
		if (m_livecast) {
			String rtmpurl = "rtmp://publish.3tee.com.cn/ucloud/12345678";
			m_livecast->startPublish(rtmpurl);
		}
		//m_video->setVideoBitrate(fromId, 500 * 1000, 800 * 1000);
	}
}
void CVideoPage::onUnpublishLocalResult(Result result, const DeviceId& fromId) {
	if (RT_SUCCEEDED(result)) {
		RemoveVideoRender(m_video->getOwnerId(fromId), fromId);
		_room_wnd->SetVideoState(false);
	}
	if (m_livecast) {
		m_livecast->stopPublish();
	}
}
///
void CVideoPage::onCameraPlugin(const Camera& item) {
	qDebug("onCameraPlugin,id=%s,name=%s", item.id.c_str(), item.name.c_str());
}
void CVideoPage::onCameraPlugout(const Camera& item) {
	qDebug("onCameraPlugout,id=%s,name=%s", item.id.c_str(), item.name.c_str());
}

///
bool CVideoPage::hasRecord(const DeviceId& deviceId) {
	if (!_room_wnd->GetRoomHandle()->getRoomInfo().isMcu()) {
		return false;
	}
	return true;
}
bool CVideoPage::hasLive(const DeviceId& deviceId) {
	if (!_room_wnd->GetRoomHandle()->getRoomInfo().isMcu()) {
		return false;
	}
	const Camera& camera = m_video->getCameraDevice()->getCamera(deviceId);
	VideoCodec codec = camera.publishedQualities.getStreamCodec(stream_main);
	if (codec_h264 != codec && camera.publishedQualities.hasAssist1()) {
		codec = camera.publishedQualities.getStreamCodec(stream_assist1);
	}
	if (codec_h264 != codec && camera.publishedQualities.hasAssist2()) {
		codec = camera.publishedQualities.getStreamCodec(stream_assist2);
	}
	return (codec_h264 == codec);
}

RecordVideoType CVideoPage::getH264Stream(const QString& videoId) {
	DeviceId sid = videoId.toUtf8().data();
	const Camera& camera = m_video->getCameraDevice()->getCamera(sid);
	VideoCodec codec = camera.publishedQualities.getStreamCodec(stream_main);
	if (codec_h264 == codec) {
		return rv_main;
	}
	if (camera.publishedQualities.hasAssist1()) {
		codec = camera.publishedQualities.getStreamCodec(stream_assist1);
		if (codec_h264 == codec) {
			return rv_ass1;
		}
	}
	if (camera.publishedQualities.hasAssist2()) {
		codec = camera.publishedQualities.getStreamCodec(stream_assist2);
		if (codec_h264 == codec) {
			return rv_ass2;
		}
	}
	return rv_none;
}
bool CVideoPage::recordStart(const QString& videoId) {
	qDebug() << "recordStart, videoId=" << videoId;
	DeviceId sid = videoId.toUtf8().data();
	sid = m_video->getOwnerId(sid);
	User user = m_video->getUserManager()->getUser(sid);
	if (!user.isValid()) {
		return false;
	}
	if (_currRecordInfo.isValid()) {
		qDebug() << "recordStart, user has begin record. recordId=" << _currRecordInfo.id.c_str();
		return false;
	}
	_currRecordInfo.roomId = CRoomProfile::Instance()->GetRoomID().toUtf8().constData();
	_currRecordInfo.userId = user.userId;
	_currRecordInfo.name = user.userName + "_rec_" + QDateTime::currentDateTime().toString(Qt::ISODate).toUtf8().constData();
	_currRecordInfo.audioType = ra_user_single;
	_currRecordInfo.videoType = rv_main;
	VideoCodec codec;
	IAVDEngine::Instance()->getOption(eo_video_codec_priority, &codec);
	_currRecordInfo.fileType = (codec_vp8 == codec) ? mt_webm : mt_mp4;
	uint32 callid = 0;
	Result ret = m_record->createUserRecord(callid, _currRecordInfo);
	return RT_SUCCEEDED(ret);
}
bool CVideoPage::recordStop(const QString& recordid) {
	qDebug() << "recordStop, recordid=" << recordid;
	if (recordid.isEmpty()) {
		return true;
	}
	uint32 callid = 0;
	LiveId sliveId = recordid.toUtf8().data();
	Result ret = m_record->stopRecord(callid, sliveId);
	CVideoRenderWnd* view = getViewByRecordId(recordid);
	if (view) {
		QString qsempty;
		view->setRecordId(qsempty);
	}
	_currRecordInfo = RecordInfo();
	return RT_SUCCEEDED(ret);
}
bool CVideoPage::liveStart(const QString& videoId) {
	qDebug() << "liveStart, videoId=" << videoId;
	DeviceId sid = videoId.toUtf8().data();
	sid = m_video->getOwnerId(sid);
	User user = m_video->getUserManager()->getUser(sid);
	if (!user.isValid()) {
		return false;
	}
	if (_currLiveInfo.isValid()) {
		qDebug() << "liveStart, user has begin live. liveId=" << _currLiveInfo.id.c_str();
		return false;
	}
	_currLiveInfo.roomId = CRoomProfile::Instance()->GetRoomID().toUtf8().constData();
	_currLiveInfo.userId = user.userId;
	_currLiveInfo.name = user.userName + "_liv_" + QDateTime::currentDateTime().toString(Qt::ISODate).toUtf8().constData();
	_currLiveInfo.audioType = ra_user_single;
	_currLiveInfo.videoType = getH264Stream(videoId);
#ifdef _DEBUG
	//_currLiveInfo.publishurl = "rtmp://publish.3tee.com.cn/ucloud/$livename";
	//_currLiveInfo.hlsurl = "http://hls.3tee.com.cn/ucloud/$livename/playlist.m3u8";
	//_currLiveInfo.rtmpurl = "rtmp://rtmp.3tee.com.cn/ucloud/$livename";
#endif
	uint32 callid = 0;
	Result ret = m_live->createUserLive(callid, _currLiveInfo);
	return RT_SUCCEEDED(ret);
}
bool CVideoPage::liveStop(const QString& liveid) {
	qDebug() << "liveStop, liveid=" << liveid;
	if (liveid.isEmpty()) {
		return true;
	}
	uint32 callid = 0;
	LiveId sliveId = liveid.toUtf8().data();
	Result ret = m_live->deleteLive(callid, sliveId);
	CVideoRenderWnd* view = getViewByLiveId(liveid);
	if (view) {
		QString qsempty;
		view->setLiveId(qsempty);
	}
	_currLiveInfo = LiveInfo();
	return RT_SUCCEEDED(ret);
}
void CVideoPage::StopLiveRecord() {
	uint32 callid = 0;
	if (_currLiveInfo.isValid()) {
		if (m_live) {
			m_live->deleteLive(callid, _currLiveInfo.id);
		}
		_currLiveInfo = LiveInfo();
	}
	if (_currRecordInfo.isValid()) {
		if (m_record) {
			m_record->stopRecord(callid, _currRecordInfo.id);
		}
		_currRecordInfo = RecordInfo();
	}
}
CVideoRenderWnd* CVideoPage::getViewByLiveId(const QString& id) {
	ID2ViewType::iterator it = _video_id2view.begin();
	for (; _video_id2view.end() != it; ++it) {
		if (id == it->second->getLiveId()) {
			return it->second;
		}
	}
	return NULL;
}
CVideoRenderWnd* CVideoPage::getViewByRecordId(const QString& id) {
	ID2ViewType::iterator it = _video_id2view.begin();
	for (; _video_id2view.end() != it; ++it) {
		if (id == it->second->getRecordId()) {
			return it->second;
		}
	}
	return NULL;
}
CVideoRenderWnd* CVideoPage::getViewByUserId(const UserId& id) {
	ID2ViewType::iterator it = _video_id2view.begin();
	for (; _video_id2view.end() != it; ++it) {
		if (id == m_video->getOwnerId(it->first.toUtf8().data())) {
			return it->second;
		}
	}
	return NULL;
}
void CVideoPage::videoOptionChanged() {
	if (NULL == m_video || NULL == m_video->getUserManager()) {
		return;
	}
	CVideoRenderWnd* item = getViewByUserId(m_video->getUserManager()->getSelfUserId());
	if (NULL == item) {
		return;
	}
	MessageBoxFactory::Get()->CreateMessageBox(MessageBoxFactory::ChildAutoBox, this, QColor(), "",
		QSTRING("ƵѡѸı䣬·Ƶֱ¼ƣ"),
		QRect(10, rect().height() - 50, rect().width() - 20, 50), 5000);
}
///
void CVideoPage::onWorkingRooms(uint32 callId, Result result, uint32 total, uint32 begin, const RoomInfosType& items) {
}
void CVideoPage::onPublishedVideos(uint32 callId, Result result, const RoomId& roomId, const VideoDevicesType& items) {
}
void CVideoPage::onCreateRecord(uint32 callId, Result result, const RecordInfo& info) {
	qDebug() << "onCreateRecord, result=" << result << ",recordId=" << info.id.c_str();
}
void CVideoPage::onSelectMedia4Record(uint32 callId, Result result, const RecordId& recordId, const DeviceId& selectedVideo) {
	qDebug() << "onSelectMedia4Record, result=" << result << ",recordId=" << recordId.c_str();
}
void CVideoPage::onCreateUserRecord(uint32 callId, Result result, const RecordInfo& info) {
	qDebug() << "onCreateUserRecord, result=" << result << ",recordId=" << info.id.c_str();
	CVideoRenderWnd* view = getViewByUserId(_currRecordInfo.userId);
	if (view) {
		QString recordId = QString::fromUtf8(info.id.c_str());
		view->setRecordId(recordId);
	}
	if (RT_SUCCEEDED(result)) {
		_currRecordInfo = info;
	}
	else {
		MessageBoxFactory::Get()->CreateMessageBox(MessageBoxFactory::ChildAutoBox, this, QColor(), "", QSTRING("¼ʧܣϵͳ´"),
			QRect(10, rect().height() - 50, rect().width() - 20, 50), 3000);
	}
}
void CVideoPage::onStopRecord(uint32 callId, Result result, const RecordId& recordId) {
	qDebug() << "onStopRecord, result=" << result << ",recordId=" << recordId.c_str();
	_currRecordInfo.status = rs_availble;
}
void CVideoPage::onRecordInfo(uint32 callId, Result result, const RecordInfo& info) {
	qDebug() << "onRecordInfo, result=" << result << ",recordId=" << info.id.c_str();
}
void CVideoPage::onRecordInfos(uint32 callId, Result result, uint32 total, uint32 begin, const RecordInfosType& items) {
	qDebug() << "onRecordInfos, result=" << result << ",total=" << total << ",begin=" << begin << ",count=" << items.size();
	if (_room_wnd->GetInviteDlg()) {
		_room_wnd->GetInviteDlg()->updateRecordInfos(items);
	}
}
void CVideoPage::onDeleteRecord(uint32 callId, Result result, const RecordId& recordId) {
	qDebug() << "onDeleteRecord, result=" << result << ",recordId=" << recordId.c_str();
	_currRecordInfo.status = rs_removed;
}
///
void CVideoPage::onCreateLive(uint32 callId, Result result, const LiveInfo& info) {
	qDebug() << "onCreateLive, result=" << result << ",liveId=" << info.id.c_str();
}
void CVideoPage::onSelectMedia4Live(uint32 callId, Result result, const LiveId& liveId, const DeviceId& selectedVideo) {
	qDebug() << "onSelectMedia4Live, result=" << result << ",liveId=" << liveId.c_str();
}
void CVideoPage::onCreateUserLive(uint32 callId, Result result, const LiveInfo& info) {
	qDebug() << "onCreateUserLive, result=" << result << ",liveId=" << info.id.c_str();
	CVideoRenderWnd* view = getViewByUserId(_currLiveInfo.userId);
	if (view) {
		QString recordId = QString::fromUtf8(info.id.c_str());
		view->setLiveId(recordId);
	}
	if (RT_SUCCEEDED(result)) {
		_currLiveInfo = info;
	}
	else {
		MessageBoxFactory::Get()->CreateMessageBox(MessageBoxFactory::ChildAutoBox, this, QColor(), "", QSTRING("ֱʧܣϵͳ´"),
			QRect(10, rect().height() - 50, rect().width() - 20, 50), 3000);
	}
	//m_live->getLiveInfo(callId, info.id);
}
void CVideoPage::onStopLive(uint32 callId, Result result, const LiveId& liveId) {
	qDebug() << "onStopLive, result=" << result << ",liveId=" << liveId.c_str();
	_currLiveInfo.status = ls_stopped;
	//m_live->deleteLive(callId, recordId);
}
void CVideoPage::onLiveInfo(uint32 callId, Result result, const LiveInfo& info) {
	qDebug() << "onLiveInfo, result=" << result << ",liveId=" << info.id.c_str();
	//m_live->stopLive(callId, info.id);
}
void CVideoPage::onLiveInfos(uint32 callId, Result result, uint32 total, uint32 begin, const LiveInfosType& items) {
	qDebug() << "onLiveInfos, result=" << result << ",total=" << total << ",begin=" << begin << ",count=" << items.size();
	if (_room_wnd->GetInviteDlg()) {
		_room_wnd->GetInviteDlg()->updateLiveInfos(items);
	}
}
void CVideoPage::onDeleteLive(uint32 callId, Result result, const LiveId& liveId) {
	qDebug() << "onDeleteLive, result=" << result << ",liveId=" << liveId.c_str();
	_currLiveInfo.status = ls_removed;
}
///
void CVideoPage::onLivecastStatus(LivecastStatus status) {
	qDebug() << "onLivecastStatus, status=" << status;
}

